/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Bot.hh"

// Konstruktor.
Bot::Bot()
: mNumAcceptInRow(0), mNumDeclineInRow(0), mPoints(0)
{
    // Standardstrategie festlegen
    setStrategy( "accept" );
}

// Setzt die Strategie fuer einen Bot.
bool Bot::setStrategy( const std::string& strategy )
{
    bool retValue = false;

    // Pruefen, ob Strategie gueltig ist.
    // Falls nicht, liefern wir false zurueck.
    if ( ( std::string("accept")   == strategy ) ||
         ( std::string("decline")  == strategy ) ||
         ( std::string("variable") == strategy ) )
    {
        mStrategy = strategy;
        retValue = true;
    }

    return retValue;
}


// Akzeptiert oder lehnt ein Angebot ab.
bool Bot::acceptOffer( const int value )
{
    bool retValue = false;

    if ( ( std::string("accept") == mStrategy ) ||
         ( mNumAcceptInRow >= 3 ) )
    {
        // Immer annehmen.
        retValue = true;
    }
    else if ( ( std::string("decline") == mStrategy ) ||
              ( mNumDeclineInRow >= 3 ) )
    {
        // Immer ablehnen.
        retValue = false;
    }
    else if ( std::string("variable") == mStrategy )
    {
        if ( value >= 200 )
        {
            retValue = true;

            // Wir merken uns, wie oft wir ein großes Angebot
            // angenommen haben.
            if ( value > 700 )
            {
                mNumAcceptInRow++;
            }
            else
            {
                mNumAcceptInRow = 0;
            }
            mNumDeclineInRow = 0;
        }
        else
        {
            // Wert ist zu klein, das merken wir uns
            retValue = false;
            mNumDeclineInRow++;
            mNumAcceptInRow = 0;
        }
    }

    if ( retValue )
    {
        // Wenn wir das Angebot annehmen, speichern
        // wir, wie viele Punkte wir erhalten haben.
        mPoints += value;
    }

    return retValue;
}

// Gibt die gesammelten Punkte zurueck.
int Bot::getPoints() const
{
    return mPoints;
}
